@@ -1,52 +1,110 @@
 from codecs import utf_8_decode
 from codecs import utf_8_encode
+from hashlib import md5
 import datetime
 import re
-import time
-
-from paste.auth import auth_tkt
-from paste.request import get_cookies
+import time as time_mod
+import urllib
 
 from zope.interface import implements
 
 from pyramid.interfaces import IAuthenticationPolicy
+from pyramid.interfaces import IDebugLogger
 
-from pyramid.request import add_global_response_headers
 from pyramid.security import Authenticated
 from pyramid.security import Everyone
 
-
 VALID_TOKEN = re.compile(r"^[A-Za-z][A-Za-z0-9+_-]*$")
 
 class CallbackAuthenticationPolicy(object):
     """ Abstract class """
+
+    debug = False
+    callback = None
+
+    def _log(self, msg, methodname, request):
+        logger = request.registry.queryUtility(IDebugLogger)
+        if logger:
+            cls = self.__class__
+            classname = cls.__module__ + '.' + cls.__name__
+            methodname = classname + '.' + methodname
+            logger.debug(methodname + ': ' + msg)
+
     def authenticated_userid(self, request):
+        debug = self.debug
         userid = self.unauthenticated_userid(request)
         if userid is None:
+            debug and self._log(
+                'call to unauthenticated_userid returned None; returning None',
+                'authenticated_userid',
+                request)
             return None
         if self.callback is None:
+            debug and self._log(
+                'there was no groupfinder callback; returning %r' % (userid,),
+                'authenticated_userid',
+                request)
             return userid
-        if self.callback(userid, request) is not None: # is not None!
+        callback_ok = self.callback(userid, request)
+        if callback_ok is not None: # is not None!
+            debug and self._log(
+                'groupfinder callback returned %r; returning %r' % (
+                    callback_ok, userid),
+                'authenticated_userid',
+                request
+                )
             return userid
+        debug and self._log(
+            'groupfinder callback returned None; returning None',
+            'authenticated_userid',
+            request
+            )
 
     def effective_principals(self, request):
+        debug = self.debug
         effective_principals = [Everyone]
         userid = self.unauthenticated_userid(request)
         if userid is None:
+            debug and self._log(
+                'authenticated_userid returned %r; returning %r' % (
+                    userid, effective_principals),
+                'effective_principals',
+                request
+                )
             return effective_principals
         if self.callback is None:
+            debug and self._log(
+                'groupfinder callback is None, so groups is []',
+                'effective_principals',
+                request)
             groups = []
         else:
             groups = self.callback(userid, request)
+            debug and self._log(
+                'groupfinder callback returned %r as groups' % (groups,),
+                'effective_principals',
+                request)
         if groups is None: # is None!
+            debug and self._log(
+                'returning effective principals: %r' % (
+                    effective_principals,),
+                'effective_principals',
+                request
+                )
             return effective_principals
+
         effective_principals.append(Authenticated)
         effective_principals.append(userid)
         effective_principals.extend(groups)
 
+        debug and self._log(
+            'returning effective principals: %r' % (
+                effective_principals,),
+            'effective_principals',
+            request
+             )
         return effective_principals
 
-
 class RepozeWho1AuthenticationPolicy(CallbackAuthenticationPolicy):
     """ A :app:`Pyramid` :term:`authentication policy` which
     obtains data from the :mod:`repoze.who` 1.X WSGI 'API' (the
@@ -61,12 +119,15 @@ class RepozeWho1AuthenticationPolicy(CallbackAuthenticationPolicy):
 
     ``callback``
 
-        Default: ``None``.  A callback passed the :mod:`repoze.who`
-        identity and the :term:`request`, expected to return ``None``
-        if the user represented by the identity doesn't exist or a
-        sequence of group identifiers (possibly empty) if the user
-        does exist.  If ``callback`` is None, the userid will be
-        assumed to exist with no groups.
+        Default: ``None``.  A callback passed the :mod:`repoze.who` identity
+        and the :term:`request`, expected to return ``None`` if the user
+        represented by the identity doesn't exist or a sequence of principal
+        identifiers (possibly empty) representing groups if the user does
+        exist.  If ``callback`` is None, the userid will be assumed to exist
+        with no group principals.
+
+    Objects of this class implement the interface described by
+    :class:`pyramid.interfaces.IAuthenticationPolicy`.
     """
     implements(IAuthenticationPolicy)
 
@@ -146,16 +207,27 @@ class RemoteUserAuthenticationPolicy(CallbackAuthenticationPolicy):
     ``callback``
 
         Default: ``None``.  A callback passed the userid and the request,
-        expected to return None if the userid doesn't exist or a sequence
-        of group identifiers (possibly empty) if the user does exist.
-        If ``callback`` is None, the userid will be assumed to exist with no
-        groups.
+        expected to return None if the userid doesn't exist or a sequence of
+        principal identifiers (possibly empty) representing groups if the
+        user does exist.  If ``callback`` is None, the userid will be assumed
+        to exist with no group principals.
+
+    ``debug``
+
+        Default: ``False``.  If ``debug`` is ``True``, log messages to the
+        Pyramid debug logger about the results of various authentication
+        steps.  The output from debugging is useful for reporting to maillist
+        or IRC channels when asking for support.
+
+    Objects of this class implement the interface described by
+    :class:`pyramid.interfaces.IAuthenticationPolicy`.
     """
     implements(IAuthenticationPolicy)
 
-    def __init__(self, environ_key='REMOTE_USER', callback=None):
+    def __init__(self, environ_key='REMOTE_USER', callback=None, debug=False):
         self.environ_key = environ_key
         self.callback = callback
+        self.debug = debug
 
     def unauthenticated_userid(self, request):
         return request.environ.get(self.environ_key)
@@ -181,9 +253,9 @@ class AuthTktAuthenticationPolicy(CallbackAuthenticationPolicy):
 
        Default: ``None``.  A callback passed the userid and the
        request, expected to return ``None`` if the userid doesn't
-       exist or a sequence of group identifiers (possibly empty) if
+       exist or a sequence of principal identifiers (possibly empty) if
        the user does exist.  If ``callback`` is ``None``, the userid
-       will be assumed to exist with no groups.  Optional.
+       will be assumed to exist with no principals.  Optional.
 
     ``cookie_name``
 
@@ -210,18 +282,22 @@ class AuthTktAuthenticationPolicy(CallbackAuthenticationPolicy):
 
     ``reissue_time``
 
-       Default: ``None``.  If this parameter is set, it represents the
-       number of seconds that must pass before an authentication token
-       cookie is reissued.  The duration is measured as the number of
-       seconds since the last auth_tkt cookie was issued and 'now'.
-       If the ``timeout`` value is ``None``, this parameter has no
-       effect.  If this parameter is provided, and the value of
-       ``timeout`` is not ``None``, the value of ``reissue_time`` must
-       be smaller than value of ``timeout``.  A good rule of thumb: if
-       you want auto-reissued cookies: set this to the ``timeout``
-       value divided by ten.  If this value is ``0``, a new ticket
-       cookie will be reissued on every request which needs
-       authentication. Optional.
+       Default: ``None``.  If this parameter is set, it represents the number
+       of seconds that must pass before an authentication token cookie is
+       automatically reissued as the result of a request which requires
+       authentication.  The duration is measured as the number of seconds
+       since the last auth_tkt cookie was issued and 'now'.  If this value is
+       ``0``, a new ticket cookie will be reissued on every request which
+       requires authentication.
+
+       A good rule of thumb: if you want auto-expired cookies based on
+       inactivity: set the ``timeout`` value to 1200 (20 mins) and set the
+       ``reissue_time`` value to perhaps a tenth of the ``timeout`` value
+       (120 or 2 mins).  It's nonsensical to set the ``timeout`` value lower
+       than the ``reissue_time`` value, as the ticket will never be reissued
+       if so.  However, such a configuration is not explicitly prevented.
+
+       Optional.
 
     ``max_age``
 
@@ -253,6 +329,16 @@ class AuthTktAuthenticationPolicy(CallbackAuthenticationPolicy):
        Default: ``True``. An auth_tkt cookie will be generated for the
        wildcard domain.
        Optional.
+
+    ``debug``
+
+        Default: ``False``.  If ``debug`` is ``True``, log messages to the
+        Pyramid debug logger about the results of various authentication
+        steps.  The output from debugging is useful for reporting to maillist
+        or IRC channels when asking for support.
+
+    Objects of this class implement the interface described by
+    :class:`pyramid.interfaces.IAuthenticationPolicy`.
     """
     implements(IAuthenticationPolicy)
     def __init__(self,
@@ -267,6 +353,7 @@ class AuthTktAuthenticationPolicy(CallbackAuthenticationPolicy):
                  path="/",
                  http_only=False,
                  wild_domain=True,
+                 debug=False,
                  ):
         self.cookie = AuthTktCookieHelper(
             secret,
@@ -281,6 +368,7 @@ class AuthTktAuthenticationPolicy(CallbackAuthenticationPolicy):
             wild_domain=wild_domain,
             )
         self.callback = callback
+        self.debug = debug
 
     def unauthenticated_userid(self, request):
         result = self.cookie.identify(request)
@@ -300,6 +388,140 @@ def b64encode(v):
 def b64decode(v):
     return v.decode('base64')
 
+# this class licensed under the MIT license (stolen from Paste)
+class AuthTicket(object):
+    """
+    This class represents an authentication token.  You must pass in
+    the shared secret, the userid, and the IP address.  Optionally you
+    can include tokens (a list of strings, representing role names),
+    'user_data', which is arbitrary data available for your own use in
+    later scripts.  Lastly, you can override the cookie name and
+    timestamp.
+
+    Once you provide all the arguments, use .cookie_value() to
+    generate the appropriate authentication ticket.
+
+    CGI usage::
+
+        token = auth_tkt.AuthTick('sharedsecret', 'username',
+            os.environ['REMOTE_ADDR'], tokens=['admin'])
+        print 'Status: 200 OK'
+        print 'Content-type: text/html'
+        print token.cookie()
+        print
+        ... redirect HTML ...
+
+    Webware usage::
+
+        token = auth_tkt.AuthTick('sharedsecret', 'username',
+            self.request().environ()['REMOTE_ADDR'], tokens=['admin'])
+        self.response().setCookie('auth_tkt', token.cookie_value())
+    """
+
+    def __init__(self, secret, userid, ip, tokens=(), user_data='',
+                 time=None, cookie_name='auth_tkt',
+                 secure=False):
+        self.secret = secret
+        self.userid = userid
+        self.ip = ip
+        self.tokens = ','.join(tokens)
+        self.user_data = user_data
+        if time is None:
+            self.time = time_mod.time()
+        else:
+            self.time = time
+        self.cookie_name = cookie_name
+        self.secure = secure
+
+    def digest(self):
+        return calculate_digest(
+            self.ip, self.time, self.secret, self.userid, self.tokens,
+            self.user_data)
+
+    def cookie_value(self):
+        v = '%s%08x%s!' % (self.digest(), int(self.time),
+                           urllib.quote(self.userid))
+        if self.tokens:
+            v += self.tokens + '!'
+        v += self.user_data
+        return v
+
+# this class licensed under the MIT license (stolen from Paste)
+class BadTicket(Exception):
+    """
+    Exception raised when a ticket can't be parsed.  If we get far enough to
+    determine what the expected digest should have been, expected is set.
+    This should not be shown by default, but can be useful for debugging.
+    """
+    def __init__(self, msg, expected=None):
+        self.expected = expected
+        Exception.__init__(self, msg)
+
+# this function licensed under the MIT license (stolen from Paste)
+def parse_ticket(secret, ticket, ip):
+    """
+    Parse the ticket, returning (timestamp, userid, tokens, user_data).
+
+    If the ticket cannot be parsed, a ``BadTicket`` exception will be raised
+    with an explanation.
+    """
+    ticket = ticket.strip('"')
+    digest = ticket[:32]
+    try:
+        timestamp = int(ticket[32:40], 16)
+    except ValueError, e:
+        raise BadTicket('Timestamp is not a hex integer: %s' % e)
+    try:
+        userid, data = ticket[40:].split('!', 1)
+    except ValueError:
+        raise BadTicket('userid is not followed by !')
+    userid = urllib.unquote(userid)
+    if '!' in data:
+        tokens, user_data = data.split('!', 1)
+    else: # pragma: no cover (never generated)
+        # @@: Is this the right order?
+        tokens = ''
+        user_data = data
+
+    expected = calculate_digest(ip, timestamp, secret,
+                                userid, tokens, user_data)
+
+    if expected != digest:
+        raise BadTicket('Digest signature is not correct',
+                        expected=(expected, digest))
+
+    tokens = tokens.split(',')
+
+    return (timestamp, userid, tokens, user_data)
+
+# this function licensed under the MIT license (stolen from Paste)
+def calculate_digest(ip, timestamp, secret, userid, tokens, user_data):
+    secret = maybe_encode(secret)
+    userid = maybe_encode(userid)
+    tokens = maybe_encode(tokens)
+    user_data = maybe_encode(user_data)
+    digest0 = md5(
+        encode_ip_timestamp(ip, timestamp) + secret + userid + '\0'
+        + tokens + '\0' + user_data).hexdigest()
+    digest = md5(digest0 + secret).hexdigest()
+    return digest
+
+# this function licensed under the MIT license (stolen from Paste)
+def encode_ip_timestamp(ip, timestamp):
+    ip_chars = ''.join(map(chr, map(int, ip.split('.'))))
+    t = int(timestamp)
+    ts = ((t & 0xff000000) >> 24,
+          (t & 0xff0000) >> 16,
+          (t & 0xff00) >> 8,
+          t & 0xff)
+    ts_chars = ''.join(map(chr, ts))
+    return ip_chars + ts_chars
+
+def maybe_encode(s, encoding='utf8'):
+    if isinstance(s, unicode):
+        s = s.encode(encoding)
+    return s
+
 EXPIRE = object()
 
 class AuthTktCookieHelper(object):
@@ -309,7 +531,9 @@ class AuthTktCookieHelper(object):
     :class:`pyramid.authentication.AuthTktAuthenticationPolicy` for the
     meanings of the constructor arguments.
     """
-    auth_tkt = auth_tkt # for tests
+    parse_ticket = staticmethod(parse_ticket) # for tests
+    AuthTicket = AuthTicket # for tests
+    BadTicket = BadTicket # for tests
     now = None # for tests
 
     userid_type_decoders = {
@@ -334,9 +558,6 @@ class AuthTktCookieHelper(object):
         self.include_ip = include_ip
         self.secure = secure
         self.timeout = timeout
-        if reissue_time is not None and timeout is not None:
-            if reissue_time > timeout:
-                raise ValueError('reissue_time must be lower than timeout')
         self.reissue_time = reissue_time
         self.max_age = max_age
         self.http_only = http_only
@@ -366,6 +587,18 @@ class AuthTktCookieHelper(object):
 
         cur_domain = environ.get('HTTP_HOST', environ.get('SERVER_NAME'))
 
+        # While Chrome, IE, and Firefox can cope, Opera (at least) cannot
+        # cope with a port number in the cookie domain when the URL it
+        # receives the cookie from does not also have that port number in it
+        # (e.g via a proxy).  In the meantime, HTTP_HOST is sent with port
+        # number, and neither Firefox nor Chrome do anything with the
+        # information when it's provided in a cookie domain except strip it
+        # out.  So we strip out any port number from the cookie domain
+        # aggressively to avoid problems.  See also
+        # https://github.com/Pylons/pyramid/issues/131
+        if ':' in cur_domain:
+            cur_domain = cur_domain.split(':', 1)[0]
+
         cookies = [
             ('Set-Cookie', '%s="%s"; Path=%s%s%s' % (
             self.cookie_name, value, self.path, max_age, self.static_flags)),
@@ -386,10 +619,9 @@ class AuthTktCookieHelper(object):
         """ Return a dictionary with authentication information, or ``None``
         if no valid auth_tkt is attached to ``request``"""
         environ = request.environ
-        cookies = get_cookies(environ)
-        cookie = cookies.get(self.cookie_name)
+        cookie = request.cookies.get(self.cookie_name)
 
-        if cookie is None or not cookie.value:
+        if cookie is None:
             return None
 
         if self.include_ip:
@@ -398,17 +630,18 @@ class AuthTktCookieHelper(object):
             remote_addr = '0.0.0.0'
         
         try:
-            timestamp, userid, tokens, user_data = self.auth_tkt.parse_ticket(
-                self.secret, cookie.value, remote_addr)
-        except self.auth_tkt.BadTicket:
+            timestamp, userid, tokens, user_data = self.parse_ticket(
+                self.secret, cookie, remote_addr)
+        except self.BadTicket:
             return None
 
         now = self.now # service tests
 
         if now is None: 
-            now = time.time()
+            now = time_mod.time()
 
         if self.timeout and ( (timestamp + self.timeout) < now ):
+            # the auth_tkt data has expired
             return None
 
         userid_typename = 'userid_type:'
@@ -421,14 +654,18 @@ class AuthTktCookieHelper(object):
                     userid = decoder(userid)
 
         reissue = self.reissue_time is not None
-            
-        if not hasattr(request, '_authtkt_reissued'):
-            if reissue and ( (now - timestamp) > self.reissue_time):
+
+        if reissue and not hasattr(request, '_authtkt_reissued'):
+            if ( (now - timestamp) > self.reissue_time ):
                 # work around https://github.com/Pylons/pyramid/issues#issue/108
                 tokens = filter(None, tokens)
                 headers = self.remember(request, userid, max_age=self.max_age,
                                         tokens=tokens)
-                add_global_response_headers(request, headers)
+                def reissue_authtkt(request, response):
+                    if not hasattr(request, '_authtkt_reissue_revoked'):
+                        for k, v in headers:
+                            response.headerlist.append((k, v))
+                request.add_response_callback(reissue_authtkt)
                 request._authtkt_reissued = True
 
         environ['REMOTE_USER_TOKENS'] = tokens
@@ -446,6 +683,7 @@ class AuthTktCookieHelper(object):
         """ Return a set of expires Set-Cookie headers, which will destroy
         any existing auth_tkt cookie when attached to a response"""
         environ = request.environ
+        request._authtkt_reissue_revoked = True
         return self._get_cookies(environ, '', max_age=EXPIRE)
     
     def remember(self, request, userid, max_age=None, tokens=()):
@@ -455,8 +693,10 @@ class AuthTktCookieHelper(object):
         ``max_age``
           The max age of the auth_tkt cookie, in seconds.  When this value is
           set, the cookie's ``Max-Age`` and ``Expires`` settings will be set,
-          allowing the auth_tkt cookie to last between browser sessions.
-          Default: ``None``.
+          allowing the auth_tkt cookie to last between browser sessions.  If
+          this value is ``None``, the ``max_age`` value provided to the
+          helper itself will be used as the ``max_age`` value.  Default:
+          ``None``.
 
         ``tokens``
           A sequence of strings that will be placed into the auth_tkt tokens
@@ -465,7 +705,9 @@ class AuthTktCookieHelper(object):
           Tokens are available in the returned identity when an auth_tkt is
           found in the request and unpacked.  Default: ``()``.
         """
-        max_age = max_age or self.max_age
+        if max_age is None:
+            max_age = self.max_age
+
         environ = request.environ
 
         if self.include_ip:
@@ -476,16 +718,25 @@ class AuthTktCookieHelper(object):
         user_data = ''
 
         encoding_data = self.userid_type_encoders.get(type(userid))
+
         if encoding_data:
             encoding, encoder = encoding_data
             userid = encoder(userid)
             user_data = 'userid_type:%s' % encoding
         
         for token in tokens:
+            if isinstance(token, unicode):
+                try:
+                    token = token.encode('ascii')
+                except UnicodeEncodeError:
+                    pass
             if not (isinstance(token, str) and VALID_TOKEN.match(token)):
                 raise ValueError("Invalid token %r" % (token,))
 
-        ticket = self.auth_tkt.AuthTicket(
+        if hasattr(request, '_authtkt_reissued'):
+            request._authtkt_reissue_revoked = True
+
+        ticket = self.AuthTicket(
             self.secret,
             userid,
             remote_addr,
@@ -496,4 +747,55 @@ class AuthTktCookieHelper(object):
 
         cookie_value = ticket.cookie_value()
         return self._get_cookies(environ, cookie_value, max_age)
-    
+
+class SessionAuthenticationPolicy(CallbackAuthenticationPolicy):
+    """ A :app:`Pyramid` authentication policy which gets its data from the
+    configured :term:`session`.  For this authentication policy to work, you
+    will have to follow the instructions in the :ref:`sessions_chapter` to
+    configure a :term:`session factory`.
+
+    Constructor Arguments
+
+    ``prefix``
+
+       A prefix used when storing the authentication parameters in the
+       session. Defaults to 'auth.'. Optional.
+
+    ``callback``
+
+       Default: ``None``.  A callback passed the userid and the
+       request, expected to return ``None`` if the userid doesn't
+       exist or a sequence of principal identifiers (possibly empty) if
+       the user does exist.  If ``callback`` is ``None``, the userid
+       will be assumed to exist with no principals.  Optional.
+
+    ``debug``
+
+        Default: ``False``.  If ``debug`` is ``True``, log messages to the
+        Pyramid debug logger about the results of various authentication
+        steps.  The output from debugging is useful for reporting to maillist
+        or IRC channels when asking for support.
+       
+    """
+    implements(IAuthenticationPolicy)
+
+    def __init__(self, prefix='auth.', callback=None, debug=False):
+        self.callback = callback
+        self.prefix = prefix or ''
+        self.userid_key = prefix + 'userid'
+        self.debug = debug
+
+    def remember(self, request, principal, **kw):
+        """ Store a principal in the session."""
+        request.session[self.userid_key] = principal
+        return []
+
+    def forget(self, request):
+        """ Remove the stored principal from the session."""
+        if self.userid_key in request.session:
+            del request.session[self.userid_key]
+        return []
+
+    def unauthenticated_userid(self, request):
+        return request.session.get(self.userid_key)
+
