@@ -1,7 +1,6 @@
 import copy
 import os
 
-from zope.configuration.xmlconfig import _clearContext
 from zope.deprecation import deprecated
 
 from zope.interface import implements
@@ -9,13 +8,15 @@ from zope.interface import Interface
 from zope.interface import alsoProvides
 
 from pyramid.interfaces import IRequest
+from pyramid.interfaces import IResponseFactory
 from pyramid.interfaces import ISecuredView
 from pyramid.interfaces import IView
 from pyramid.interfaces import IViewClassifier
 from pyramid.interfaces import ISession
 
 from pyramid.config import Configurator
-from pyramid.exceptions import Forbidden
+from pyramid.decorator import reify
+from pyramid.httpexceptions import HTTPForbidden
 from pyramid.response import Response
 from pyramid.registry import Registry
 from pyramid.security import Authenticated
@@ -23,6 +24,9 @@ from pyramid.security import Everyone
 from pyramid.security import has_permission
 from pyramid.threadlocal import get_current_registry
 from pyramid.threadlocal import manager
+from pyramid.request import DeprecatedRequestMethodsMixin
+from pyramid.request import CallbackMethodsMixin
+from pyramid.url import URLMethodsMixin
 
 _marker = object()
 
@@ -67,9 +71,9 @@ def registerDummySecurityPolicy(userid=None, groupids=(), permissive=True):
 
 deprecated('registerDummySecurityPolicy',
            'The testing.registerDummySecurityPolicy API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_securitypolicy '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerResources(resources):
     """ Registers a dictionary of :term:`resource` objects that can be
@@ -99,17 +103,17 @@ def registerResources(resources):
 
 deprecated('registerResources',
            'The testing.registerResources API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_resources '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 registerModels = registerResources
 
 deprecated('registerModels',
            'The testing.registerModels API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_resources '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerEventListener(event_iface=None):
     """ Registers an :term:`event` listener (aka :term:`subscriber`)
@@ -141,9 +145,9 @@ def registerEventListener(event_iface=None):
 
 deprecated('registerEventListener',
            'The testing.registerEventListener API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_add_subscriber '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerTemplateRenderer(path, renderer=None):
     """ Register a template renderer at ``path`` (usually a relative
@@ -169,9 +173,9 @@ def registerTemplateRenderer(path, renderer=None):
 
 deprecated('registerTemplateRenderer',
            'The testing.registerTemplateRenderer API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_add_renderer '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 # registerDummyRenderer is a deprecated alias that should never be removed
 # (too much usage in the wild)
@@ -179,9 +183,9 @@ registerDummyRenderer = registerTemplateRenderer
 
 deprecated('registerDummyRenderer',
            'The testing.registerDummyRenderer API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.testing_add_renderer '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerView(name, result='', view=None, for_=(Interface, Interface),
                  permission=None):
@@ -215,7 +219,7 @@ def registerView(name, result='', view=None, for_=(Interface, Interface),
     else:
         def _secure(context, request):
             if not has_permission(permission, context, request):
-                raise Forbidden('no permission')
+                raise HTTPForbidden('no permission')
             else:
                 return view(context, request)
         _secure.__call_permissive__ = view
@@ -226,9 +230,9 @@ def registerView(name, result='', view=None, for_=(Interface, Interface),
 
 deprecated('registerView',
            'The registerView API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.add_view '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerUtility(impl, iface=Interface, name=''):
     """ Register a ZCA utility component.
@@ -255,10 +259,10 @@ def registerUtility(impl, iface=Interface, name=''):
 
 deprecated('registerUtility',
            'The registerUtility API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.registry.registerUtility method (via '
            'e.g. "config.registry.registerUtility(..)" '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerAdapter(impl, for_=Interface, provides=Interface, name=''):
     """ Register a ZCA adapter component.
@@ -294,10 +298,10 @@ def registerAdapter(impl, for_=Interface, provides=Interface, name=''):
 
 deprecated('registerAdapter',
            'The registerAdapter API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.registry.registerAdapter method (via '
            'e.g. "config.registry.registerAdapter(..)" '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerSubscriber(subscriber, iface=Interface):
     """ Register a ZCA subscriber component.
@@ -327,9 +331,9 @@ def registerSubscriber(subscriber, iface=Interface):
 
 deprecated('registerSubscriber',
            'The testing.registerSubscriber API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.add_subscriber '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerRoute(pattern, name, factory=None):
     """ Register a new :term:`route` using a pattern
@@ -356,9 +360,9 @@ def registerRoute(pattern, name, factory=None):
 
 deprecated('registerRoute',
            'The testing.registerRoute API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.add_route '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 def registerSettings(dictarg=None, **kw):
     """Register one or more 'setting' key/value pairs.  A setting is
@@ -388,9 +392,9 @@ def registerSettings(dictarg=None, **kw):
 
 deprecated('registerSettings',
            'The testing.registerSettings API is deprecated as of '
-           'Pyramid 1.0. Instead use the'
+           'Pyramid 1.0. Instead use the '
            'pyramid.config.Configurator.add_settings '
-           'method in your unit and integration tests. ')
+           'method in your unit and integration tests.')
 
 class DummyRootFactory(object):
     __parent__ = None
@@ -617,11 +621,12 @@ class DummySession(dict):
     def get_csrf_token(self):
         return self.get('_csrft_', None)
         
-class DummyRequest(object):
-    """ A dummy request object (imitates a :term:`request` object).
+class DummyRequest(DeprecatedRequestMethodsMixin, URLMethodsMixin,
+                   CallbackMethodsMixin):
+    """ A DummyRequest object (incompletely) imitates a :term:`request` object.
 
     The ``params``, ``environ``, ``headers``, ``path``, and
-    ``cookies`` arguments correspond to their :term`WebOb`
+    ``cookies`` arguments correspond to their :term:`WebOb`
     equivalents.
 
     The ``post`` argument,  if passed, populates the request's
@@ -632,6 +637,17 @@ class DummyRequest(object):
     Extra keyword arguments are assigned as attributes of the request
     itself.
 
+    Note that DummyRequest does not have complete fidelity with a "real"
+    request.  For example, by default, the DummyRequest ``GET`` and ``POST``
+    attributes are of type ``dict``, unlike a normal Request's GET and POST,
+    which are of type ``MultiDict``. If your code uses the features of
+    MultiDict, you should either use a"real" :class:`pyramid.request.Request`
+    or adapt your DummyRequest by replacing the attributes with ``MultiDict``
+    instances.
+
+    Other similar incompatibilities exist.  If you need all the features of
+    a Request, use the :class:`pyramid.request.Request` class itself rather
+    than this class while writing tests.
     """
     implements(IRequest)
     method = 'GET'
@@ -639,8 +655,10 @@ class DummyRequest(object):
     host = 'example.com:80'
     content_length = 0
     query_string = ''
-    response_callbacks = ()
     charset = 'UTF-8'
+    script_name = ''
+    _registry = None
+
     def __init__(self, params=None, environ=None, headers=None, path='/',
                  cookies=None, post=None, **kw):
         if environ is None:
@@ -678,14 +696,26 @@ class DummyRequest(object):
         self.root = None
         self.virtual_root = None
         self.marshalled = params # repoze.monty
-        self.registry = get_current_registry()
         self.session = DummySession()
         self.__dict__.update(kw)
 
-    def add_response_callback(self, callback):
-        if not self.response_callbacks:
-            self.response_callbacks = []
-        self.response_callbacks.append(callback)
+    def _get_registry(self):
+        if self._registry is None:
+            return get_current_registry()
+        return self._registry
+
+    def _set_registry(self, registry):
+        self._registry = registry
+
+    def _del_registry(self):
+        self._registry = None
+
+    registry = property(_get_registry, _set_registry, _del_registry)
+
+    @reify
+    def response(self):
+        f =  self.registry.queryUtility(IResponseFactory, default=Response)
+        return f()
 
 def setUp(registry=None, request=None, hook_zca=True, autocommit=True,
           settings=None):
@@ -805,7 +835,6 @@ def tearDown(unhook_zca=True):
                 # however maybe somebody's using a registry we don't
                 # understand, let's not blow up
                 pass
-    _clearContext() # XXX why?
 
 def cleanUp(*arg, **kw):
     """ :func:`pyramid.testing.cleanUp` is an alias for
