@@ -17,5 +17,9 @@ class ConfiguratorTests(unittest.TestCase):
         config = self._makeOne()
         self.assertEqual(config.autocommit, True)
         
+    def test_package_is_not_None(self):
+        import pyramid
+        config = self._makeOne(package='pyramid')
+        self.assertEqual(config.package, pyramid)
 
 
