@@ -10,15 +10,162 @@ class TestTemplateRendererFactory(unittest.TestCase):
     def tearDown(self):
         cleanUp()
         
-    def _callFUT(self, path, factory):
+    def _callFUT(self, info, impl):
         from pyramid.renderers import template_renderer_factory
-        return template_renderer_factory(path, factory)
+        return template_renderer_factory(info, impl)
 
-    def test_abspath_notfound(self):
+    def test_lookup_found(self):
+        from pyramid.interfaces import IChameleonLookup
+        L = []
+        def dummy(info):
+            L.append(info)
+            return True
+        self.config.registry.registerUtility(dummy, IChameleonLookup,
+                                             name='abc')
+        class DummyInfo(object):
+            pass
+        info = DummyInfo()
+        info.registry = self.config.registry
+        info.type = 'abc'
+        result = self._callFUT(info, None)
+        self.assertEqual(result, True)
+        self.assertEqual(L, [info])
+
+    def test_lookup_miss(self):
         from pyramid.interfaces import ITemplateRenderer
+        import os
+        abspath = os.path.abspath(__file__)
+        renderer = {}
+        self.config.registry.registerUtility(
+            renderer, ITemplateRenderer, name=abspath)
+        info = DummyRendererInfo({
+            'name':abspath,
+            'package':None,
+            'registry':self.config.registry,
+            'settings':{},
+            'type':'type',
+            })
+        result = self._callFUT(info, None)
+        self.assertTrue(result is renderer)
+
+class TestChameleonRendererLookup(unittest.TestCase):
+    def setUp(self):
+        self.config = testing.setUp()
+
+    def tearDown(self):
+        testing.tearDown()
+        
+    def _makeOne(self, impl):
+        from pyramid.renderers import ChameleonRendererLookup
+        return ChameleonRendererLookup(impl, self.config.registry)
+
+    def _registerTemplateRenderer(self, renderer, name):
+        from pyramid.interfaces import ITemplateRenderer
+        self.config.registry.registerUtility(
+            renderer, ITemplateRenderer, name=name)
+
+    def test_get_spec_not_abspath_no_colon_no_package(self):
+        lookup = self._makeOne(None)
+        result = lookup.get_spec('foo', None)
+        self.assertEqual(result, 'foo')
+
+    def test_get_spec_not_abspath_no_colon_with_package(self):
+        from pyramid import tests
+        lookup = self._makeOne(None)
+        result = lookup.get_spec('foo', tests)
+        self.assertEqual(result, 'pyramid.tests:foo')
+
+    def test_get_spec_not_abspath_with_colon_no_package(self):
+        lookup = self._makeOne(None)
+        result = lookup.get_spec('fudge:foo', None)
+        self.assertEqual(result, 'fudge:foo')
+
+    def test_get_spec_not_abspath_with_colon_with_package(self):
+        from pyramid import tests
+        lookup = self._makeOne(None)
+        result = lookup.get_spec('fudge:foo', tests)
+        self.assertEqual(result, 'fudge:foo')
+
+    def test_get_spec_is_abspath_no_colon_no_package(self):
+        import os
+        lookup = self._makeOne(None)
+        spec = os.path.abspath(__file__)
+        result = lookup.get_spec(spec, None)
+        self.assertEqual(result, spec)
+
+    def test_get_spec_is_abspath_no_colon_with_path_in_package(self):
+        from pyramid import tests
+        import os
+        lookup = self._makeOne(None)
+        f = __file__
+        spec = os.path.abspath(f)
+        result = lookup.get_spec(spec, tests)
+        self.assertEqual(result, 'pyramid.tests:%s' % os.path.split(f)[-1])
+
+    def test_get_spec_is_abspath_no_colon_with_path_outside_package(self):
+        import venusian # used only because it's outside of pyramid.tests
+        import os
+        lookup = self._makeOne(None)
+        f = __file__
+        spec = os.path.abspath(f)
+        result = lookup.get_spec(spec, venusian)
+        self.assertEqual(result, spec)
+
+    def test_get_spec_is_abspath_with_colon_no_package(self):
+        import os
+        lookup = self._makeOne(None)
+        spec = os.path.join(os.path.abspath(__file__), ':foo')
+        result = lookup.get_spec(spec, None)
+        self.assertEqual(result, spec)
+
+    def test_get_spec_is_abspath_with_colon_with_path_in_package(self):
+        from pyramid import tests
+        import os
+        lookup = self._makeOne(None)
+        f = os.path.abspath(__file__)
+        spec = os.path.join(f, ':foo')
+        result = lookup.get_spec(spec, tests)
+        tail = spec.split(os.sep)[-2:]
+        self.assertEqual(result, 'pyramid.tests:%s/%s' % tuple(tail))
+
+    def test_get_spec_is_abspath_with_colon_with_path_outside_package(self):
+        import venusian # used only because it's outside of pyramid.tests
+        import os
+        lookup = self._makeOne(None)
+        spec = os.path.join(os.path.abspath(__file__), ':foo')
+        result = lookup.get_spec(spec, venusian)
+        self.assertEqual(result, spec)
+
+    def test_translate(self):
+        from pyramid.interfaces import IChameleonTranslate
+        def t(): pass
+        self.config.registry.registerUtility(t, IChameleonTranslate)
+        lookup = self._makeOne(None)
+        self.assertEqual(lookup.translate, t)
+
+    def test_debug_settings_None(self):
+        self.config.registry.settings = None
+        lookup = self._makeOne(None)
+        self.assertEqual(lookup.debug, False)
+
+    def test_debug_settings_not_None(self):
+        self.config.registry.settings = {'debug_templates':True}
+        lookup = self._makeOne(None)
+        self.assertEqual(lookup.debug, True)
+
+    def test_auto_reload_settings_None(self):
+        self.config.registry.settings = None
+        lookup = self._makeOne(None)
+        self.assertEqual(lookup.auto_reload, False)
+
+    def test_auto_reload_settings_not_None(self):
+        self.config.registry.settings = {'reload_templates':True}
+        lookup = self._makeOne(None)
+        self.assertEqual(lookup.auto_reload, True)
+
+    def test___call__abspath_path_notexists(self):
         abspath = '/wont/exist'
-        self.config.registry.registerUtility(
-            {}, ITemplateRenderer, name=abspath)
+        self._registerTemplateRenderer({}, abspath)
         info = DummyRendererInfo({
             'name':abspath,
             'package':None,
@@ -26,15 +173,14 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        self.assertRaises(ValueError, self._callFUT, info, None)
+        lookup = self._makeOne(None)
+        self.assertRaises(ValueError, lookup.__call__, info)
 
-    def test_abspath_alreadyregistered(self):
-        from pyramid.interfaces import ITemplateRenderer
+    def test___call__abspath_alreadyregistered(self):
         import os
         abspath = os.path.abspath(__file__)
         renderer = {}
-        self.config.registry.registerUtility(
-            renderer, ITemplateRenderer, name=abspath)
+        self._registerTemplateRenderer(renderer, abspath)
         info = DummyRendererInfo({
             'name':abspath,
             'package':None,
@@ -42,16 +188,15 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        result = self._callFUT(info, None)
-        self.failUnless(result is renderer)
+        lookup = self._makeOne(None)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
 
-    def test_abspath_notyetregistered(self):
-        from pyramid.interfaces import ITemplateRenderer
+    def test___call__abspath_notyetregistered(self):
         import os
         abspath = os.path.abspath(__file__)
         renderer = {}
-        self.config.registry.registerUtility(
-            renderer, ITemplateRenderer, name=abspath)
+        factory = DummyFactory(renderer)
         info = DummyRendererInfo({
             'name':abspath,
             'package':None,
@@ -59,15 +204,14 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        result = self._callFUT(info, None)
-        self.failUnless(result is renderer)
+        lookup = self._makeOne(factory)
+        result = lookup(info)
+        self.assertEqual(result, renderer)
 
-    def test_relpath_path_registered(self):
+    def test___call__relpath_path_registered(self):
         renderer = {}
-        from pyramid.interfaces import ITemplateRenderer
-        self.config.registry.registerUtility(
-            renderer, ITemplateRenderer, name='foo/bar')
         spec = 'foo/bar'
+        self._registerTemplateRenderer(renderer, spec)
         info = DummyRendererInfo({
             'name':spec,
             'package':None,
@@ -75,17 +219,15 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        result = self._callFUT(info, None)
-        self.failUnless(renderer is result)
+        lookup = self._makeOne(None)
+        result = lookup(info)
+        self.assertTrue(renderer is result)
 
-    def test_relpath_has_package_registered(self):
+    def test___call__relpath_has_package_registered(self):
         renderer = {}
-        from pyramid.interfaces import ITemplateRenderer
         import pyramid.tests
         spec = 'bar/baz'
-        self.config.registry.registerUtility(
-            renderer, ITemplateRenderer,
-            name='pyramid.tests:bar/baz')
+        self._registerTemplateRenderer(renderer, 'pyramid.tests:bar/baz')
         info = DummyRendererInfo({
             'name':spec,
             'package':pyramid.tests,
@@ -93,10 +235,11 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        result = self._callFUT(info, None)
-        self.failUnless(renderer is result)
+        lookup = self._makeOne(None)
+        result = lookup(info)
+        self.assertTrue(renderer is result)
 
-    def test_spec_notfound(self):
+    def test___call__spec_notfound(self):
         spec = 'pyramid.tests:wont/exist'
         info = DummyRendererInfo({
             'name':spec,
@@ -105,10 +248,10 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        self.assertRaises(ValueError, self._callFUT, info, None)
+        lookup = self._makeOne(None)
+        self.assertRaises(ValueError, lookup.__call__, info)
 
-    def test_spec_alreadyregistered(self):
-        from pyramid.interfaces import ITemplateRenderer
+    def test___call__spec_alreadyregistered(self):
         from pyramid import tests
         module_name = tests.__name__
         relpath = 'test_renderers.py'
@@ -121,12 +264,12 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'type':'type',
             })
         renderer = {}
-        self.config.registry.registerUtility(
-            renderer, ITemplateRenderer, name=spec)
-        result = self._callFUT(info, None)
-        self.failUnless(result is renderer)
+        self._registerTemplateRenderer(renderer, spec)
+        lookup = self._makeOne(None)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
 
-    def test_spec_notyetregistered(self):
+    def test___call__spec_notyetregistered(self):
         import os
         from pyramid import tests
         module_name = tests.__name__
@@ -141,15 +284,16 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':{},
             'type':'type',
             })
-        result = self._callFUT(info, factory)
-        self.failUnless(result is renderer)
+        lookup = self._makeOne(factory)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
         path = os.path.abspath(__file__).split('$')[0] # jython
         if path.endswith('.pyc'): # pragma: no cover
             path = path[:-1]
-        self.failUnless(factory.path.startswith(path))
+        self.assertTrue(factory.path.startswith(path))
         self.assertEqual(factory.kw, {})
 
-    def test_reload_assets_true(self):
+    def test___call__reload_assets_true(self):
         import pyramid.tests
         from pyramid.interfaces import ISettings
         from pyramid.interfaces import ITemplateRenderer
@@ -166,13 +310,14 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':settings,
             'type':'type',
             })
-        result = self._callFUT(info, factory)
-        self.failUnless(result is renderer)
+        lookup = self._makeOne(factory)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
         spec = '%s:%s' % ('pyramid.tests', 'test_renderers.py')
         self.assertEqual(reg.queryUtility(ITemplateRenderer, name=spec),
                          None)
 
-    def test_reload_assets_false(self):
+    def test___call__reload_assets_false(self):
         import pyramid.tests
         from pyramid.interfaces import ITemplateRenderer
         settings = {'reload_assets':False}
@@ -187,19 +332,24 @@ class TestTemplateRendererFactory(unittest.TestCase):
             'settings':settings,
             'type':'type',
             })
-        result = self._callFUT(info, factory)
-        self.failUnless(result is renderer)
+        lookup = self._makeOne(factory)
+        result = lookup(info)
+        self.assertTrue(result is renderer)
         spec = '%s:%s' % ('pyramid.tests', 'test_renderers.py')
         self.assertNotEqual(reg.queryUtility(ITemplateRenderer, name=spec),
                             None)
 
 class TestRendererFromName(unittest.TestCase):
     def setUp(self):
+        from zope.deprecation import __show__
+        __show__.off()
         self.config = cleanUp()
 
     def tearDown(self):
         cleanUp()
-        
+        from zope.deprecation import __show__
+        __show__.on()
+
     def _callFUT(self, path, package=None):
         from pyramid.renderers import renderer_from_name
         return renderer_from_name(path, package)
@@ -248,6 +398,12 @@ class TestRendererFromName(unittest.TestCase):
         
 
 class Test_json_renderer_factory(unittest.TestCase):
+    def setUp(self):
+        self.config = testing.setUp()
+
+    def tearDown(self):
+        testing.tearDown()
+        
     def _callFUT(self, name):
         from pyramid.renderers import json_renderer_factory
         return json_renderer_factory(name)
@@ -261,14 +417,14 @@ class Test_json_renderer_factory(unittest.TestCase):
         request = testing.DummyRequest()
         renderer = self._callFUT(None)
         renderer({'a':1}, {'request':request})
-        self.assertEqual(request.response_content_type, 'application/json')
+        self.assertEqual(request.response.content_type, 'application/json')
 
     def test_with_request_content_type_set(self):
         request = testing.DummyRequest()
-        request.response_content_type = 'text/mishmash'
+        request.response.content_type = 'text/mishmash'
         renderer = self._callFUT(None)
         renderer({'a':1}, {'request':request})
-        self.assertEqual(request.response_content_type, 'text/mishmash')
+        self.assertEqual(request.response.content_type, 'text/mishmash')
 
 class Test_string_renderer_factory(unittest.TestCase):
     def _callFUT(self, name):
@@ -297,14 +453,14 @@ class Test_string_renderer_factory(unittest.TestCase):
         request = testing.DummyRequest()
         renderer = self._callFUT(None)
         renderer(None, {'request':request})
-        self.assertEqual(request.response_content_type, 'text/plain')
+        self.assertEqual(request.response.content_type, 'text/plain')
 
     def test_with_request_content_type_set(self):
         request = testing.DummyRequest()
-        request.response_content_type = 'text/mishmash'
+        request.response.content_type = 'text/mishmash'
         renderer = self._callFUT(None)
         renderer(None, {'request':request})
-        self.assertEqual(request.response_content_type, 'text/mishmash')
+        self.assertEqual(request.response.content_type, 'text/mishmash')
 
 
 class TestRendererHelper(unittest.TestCase):
@@ -324,6 +480,25 @@ class TestRendererHelper(unittest.TestCase):
         helper = self._makeOne()
         verifyObject(IRendererInfo, helper)
 
+    def test_settings_registry_settings_is_None(self):
+        class Dummy(object):
+            settings = None
+        helper = self._makeOne(registry=Dummy)
+        self.assertEqual(helper.settings, {})
+
+    def test_settings_registry_name_is_None(self):
+        class Dummy(object):
+            settings = None
+        helper = self._makeOne(registry=Dummy)
+        self.assertEqual(helper.name, None)
+        self.assertEqual(helper.type, '')
+
+    def test_settings_registry_settings_is_not_None(self):
+        class Dummy(object):
+            settings = {'a':1}
+        helper = self._makeOne(registry=Dummy)
+        self.assertEqual(helper.settings, {'a':1})
+
     def _registerRendererFactory(self):
         from pyramid.interfaces import IRendererFactory
         def renderer(*arg):
@@ -334,16 +509,25 @@ class TestRendererHelper(unittest.TestCase):
                                              name='.foo')
         return renderer
 
+    def _registerResponseFactory(self):
+        from pyramid.interfaces import IResponseFactory
+        class ResponseFactory(object):
+            pass
+        self.config.registry.registerUtility(ResponseFactory, IResponseFactory)
+
     def test_render_to_response(self):
         self._registerRendererFactory()
+        self._registerResponseFactory()
         request = Dummy()
         helper = self._makeOne('loo.foo')
-        response = helper.render_to_response('values', 'system_values',
+        response = helper.render_to_response('values', {},
                                              request=request)
-        self.assertEqual(response.body, ('values', 'system_values'))
+        self.assertEqual(response.body[0], 'values')
+        self.assertEqual(response.body[1], {})
 
     def test_render_view(self):
         self._registerRendererFactory()
+        self._registerResponseFactory()
         request = Dummy()
         helper = self._makeOne('loo.foo')
         view = 'view'
@@ -351,13 +535,13 @@ class TestRendererHelper(unittest.TestCase):
         request = testing.DummyRequest()
         response = 'response'
         response = helper.render_view(request, response, view, context)
-        self.assertEqual(response.body,
-                         ('response',
+        self.assertEqual(response.body[0], 'response')
+        self.assertEqual(response.body[1],
                           {'renderer_info': helper,
                            'renderer_name': 'loo.foo',
                            'request': request,
                            'context': 'context',
-                           'view': 'view'})
+                           'view': 'view'}
                          )
 
     def test_render_explicit_registry(self):
@@ -373,9 +557,10 @@ class TestRendererHelper(unittest.TestCase):
         reg = DummyRegistry()
         helper = self._makeOne('loo.foo', registry=reg)
         result = helper.render('value', {})
-        self.assertEqual(result, ('value', {}))
-        self.failUnless(reg.queried)
-        self.assertEqual(reg.event._system, {})
+        self.assertEqual(result[0], 'value')
+        self.assertEqual(result[1], {})
+        self.assertTrue(reg.queried)
+        self.assertEqual(reg.event, {})
         self.assertEqual(reg.event.__class__.__name__, 'BeforeRender')
 
     def test_render_system_values_is_None(self):
@@ -391,7 +576,8 @@ class TestRendererHelper(unittest.TestCase):
                   'view':None,
                   'renderer_info':helper
                   }
-        self.assertEqual(result, ('values', system))
+        self.assertEqual(result[0], 'values')
+        self.assertEqual(result[1], system)
 
     def test_render_renderer_globals_factory_active(self):
         self._registerRendererFactory()
@@ -403,9 +589,43 @@ class TestRendererHelper(unittest.TestCase):
         result = helper.render('values', None)
         self.assertEqual(result[1]['a'], 1)
 
-    def test__make_response_with_content_type(self):
+    def test__make_response_request_is_None(self):
+        request = None
+        helper = self._makeOne('loo.foo')
+        response = helper._make_response('abc', request)
+        self.assertEqual(response.body, 'abc')
+
+    def test__make_response_request_is_None_response_factory_exists(self):
+        self._registerResponseFactory()
+        request = None
+        helper = self._makeOne('loo.foo')
+        response = helper._make_response('abc', request)
+        self.assertEqual(response.__class__.__name__, 'ResponseFactory')
+        self.assertEqual(response.body, 'abc')
+
+    def test__make_response_result_is_unicode(self):
+        from pyramid.response import Response
         request = testing.DummyRequest()
-        attrs = {'response_content_type':'text/nonsense'}
+        request.response = Response()
+        helper = self._makeOne('loo.foo')
+        la = unicode('/La Pe\xc3\xb1a', 'utf-8')
+        response = helper._make_response(la, request)
+        self.assertEqual(response.body, la.encode('utf-8'))
+
+    def test__make_response_result_is_str(self):
+        from pyramid.response import Response
+        request = testing.DummyRequest()
+        request.response = Response()
+        helper = self._makeOne('loo.foo')
+        la = unicode('/La Pe\xc3\xb1a', 'utf-8')
+        response = helper._make_response(la.encode('utf-8'), request)
+        self.assertEqual(response.body, la.encode('utf-8'))
+
+    def test__make_response_with_content_type(self):
+        from pyramid.response import Response
+        request = testing.DummyRequest()
+        request.response = Response()
+        attrs = {'_response_content_type':'text/nonsense'}
         request.__dict__.update(attrs)
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
@@ -413,8 +633,10 @@ class TestRendererHelper(unittest.TestCase):
         self.assertEqual(response.body, 'abc')
 
     def test__make_response_with_headerlist(self):
+        from pyramid.response import Response
         request = testing.DummyRequest()
-        attrs = {'response_headerlist':[('a', '1'), ('b', '2')]}
+        request.response = Response()
+        attrs = {'_response_headerlist':[('a', '1'), ('b', '2')]}
         request.__dict__.update(attrs)
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
@@ -426,8 +648,10 @@ class TestRendererHelper(unittest.TestCase):
         self.assertEqual(response.body, 'abc')
 
     def test__make_response_with_status(self):
+        from pyramid.response import Response
         request = testing.DummyRequest()
-        attrs = {'response_status':'406 You Lose'}
+        request.response = Response()
+        attrs = {'_response_status':'406 You Lose'}
         request.__dict__.update(attrs)
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
@@ -435,16 +659,20 @@ class TestRendererHelper(unittest.TestCase):
         self.assertEqual(response.body, 'abc')
 
     def test__make_response_with_charset(self):
+        from pyramid.response import Response
         request = testing.DummyRequest()
-        attrs = {'response_charset':'UTF-16'}
+        request.response = Response()
+        attrs = {'_response_charset':'UTF-16'}
         request.__dict__.update(attrs)
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.charset, 'UTF-16')
 
     def test__make_response_with_cache_for(self):
+        from pyramid.response import Response
         request = testing.DummyRequest()
-        attrs = {'response_cache_for':100}
+        request.response = Response()
+        attrs = {'_response_cache_for':100}
         request.__dict__.update(attrs)
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
@@ -453,26 +681,76 @@ class TestRendererHelper(unittest.TestCase):
     def test_with_alternate_response_factory(self):
         from pyramid.interfaces import IResponseFactory
         class ResponseFactory(object):
-            def __init__(self, result):
-                self.result = result
+            def __init__(self):
+                pass
         self.config.registry.registerUtility(ResponseFactory, IResponseFactory)
         request = testing.DummyRequest()
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.__class__, ResponseFactory)
-        self.assertEqual(response.result, 'abc')
+        self.assertEqual(response.body, 'abc')
 
     def test__make_response_with_real_request(self):
         # functional
         from pyramid.request import Request
         request = Request({})
-        attrs = {'response_status':'406 You Lose'}
-        request.__dict__.update(attrs)
+        request.registry = self.config.registry
+        request.response.status = '406 You Lose'
         helper = self._makeOne('loo.foo')
         response = helper._make_response('abc', request)
         self.assertEqual(response.status, '406 You Lose')
         self.assertEqual(response.body, 'abc')
 
+    def test_clone_noargs(self):
+        helper = self._makeOne('name', 'package', 'registry')
+        cloned_helper = helper.clone()
+        self.assertEqual(cloned_helper.name, 'name')
+        self.assertEqual(cloned_helper.package, 'package')
+        self.assertEqual(cloned_helper.registry, 'registry')
+        self.assertFalse(helper is cloned_helper)
+
+    def test_clone_allargs(self):
+        helper = self._makeOne('name', 'package', 'registry')
+        cloned_helper = helper.clone(name='name2', package='package2',
+                                     registry='registry2')
+        self.assertEqual(cloned_helper.name, 'name2')
+        self.assertEqual(cloned_helper.package, 'package2')
+        self.assertEqual(cloned_helper.registry, 'registry2')
+        self.assertFalse(helper is cloned_helper)
+
+class TestNullRendererHelper(unittest.TestCase):
+    def setUp(self):
+        self.config = cleanUp()
+
+    def tearDown(self):
+        cleanUp()
+
+    def _makeOne(self, *arg, **kw):
+        from pyramid.renderers import NullRendererHelper
+        return NullRendererHelper(*arg, **kw)
+
+    def test_instance_conforms(self):
+        from zope.interface.verify import verifyObject
+        from pyramid.interfaces import IRendererInfo
+        helper = self._makeOne()
+        verifyObject(IRendererInfo, helper)
+
+    def test_render_view(self):
+        helper = self._makeOne()
+        self.assertEqual(helper.render_view(None, True, None, None), True)
+
+    def test_render(self):
+        helper = self._makeOne()
+        self.assertEqual(helper.render(True, None, None), True)
+
+    def test_render_to_response(self):
+        helper = self._makeOne()
+        self.assertEqual(helper.render_to_response(True, None, None), True)
+
+    def test_clone(self):
+        helper = self._makeOne()
+        self.assertTrue(helper.clone() is helper)
+
 class Test_render(unittest.TestCase):
     def setUp(self):
         self.config = testing.setUp()
@@ -504,6 +782,18 @@ class Test_render(unittest.TestCase):
         renderer.assert_(a=1)
         renderer.assert_(request=request)
 
+    def test_it_with_package(self):
+        import pyramid.tests
+        renderer = self.config.testing_add_renderer(
+            'pyramid.tests:abc/def.pt')
+        renderer.string_response = 'abc'
+        request = testing.DummyRequest()
+        result = self._callFUT('abc/def.pt', dict(a=1), request=request,
+                               package=pyramid.tests)
+        self.assertEqual(result, 'abc')
+        renderer.assert_(a=1)
+        renderer.assert_(request=request)
+
 class Test_render_to_response(unittest.TestCase):
     def setUp(self):
         self.config = testing.setUp()
@@ -535,7 +825,19 @@ class Test_render_to_response(unittest.TestCase):
         self.assertEqual(response.body, 'abc')
         renderer.assert_(a=1)
         renderer.assert_(request=request)
-    
+
+    def test_it_with_package(self):
+        import pyramid.tests
+        renderer = self.config.testing_add_renderer(
+            'pyramid.tests:abc/def.pt')
+        renderer.string_response = 'abc'
+        request = testing.DummyRequest()
+        response = self._callFUT('abc/def.pt', dict(a=1), request=request,
+                                 package=pyramid.tests)
+        self.assertEqual(response.body, 'abc')
+        renderer.assert_(a=1)
+        renderer.assert_(request=request)
+
 class Test_get_renderer(unittest.TestCase):
     def setUp(self):
         self.config = testing.setUp()
@@ -545,14 +847,46 @@ class Test_get_renderer(unittest.TestCase):
 
     def _callFUT(self, renderer_name, **kw):
         from pyramid.renderers import get_renderer
-        return get_renderer(renderer_name)
+        return get_renderer(renderer_name, **kw)
 
-    def test_it(self):
+    def test_it_no_package(self):
         renderer = self.config.testing_add_renderer(
             'pyramid.tests:abc/def.pt')
         result = self._callFUT('abc/def.pt')
         self.assertEqual(result, renderer)
 
+    def test_it_with_package(self):
+        import pyramid.tests
+        renderer = self.config.testing_add_renderer(
+            'pyramid.tests:abc/def.pt')
+        result = self._callFUT('abc/def.pt', package=pyramid.tests)
+        self.assertEqual(result, renderer)
+
+class TestJSONP(unittest.TestCase):
+    def _makeOne(self, param_name='callback'):
+        from pyramid.renderers import JSONP
+        return JSONP(param_name)
+
+    def test_render_to_jsonp(self):
+        renderer_factory = self._makeOne()
+        renderer = renderer_factory(None)
+        request = testing.DummyRequest()
+        request.GET['callback'] = 'callback'
+        result = renderer({'a':'1'}, {'request':request})
+        self.assertEqual(result, 'callback({"a": "1"})')
+        self.assertEqual(request.response.content_type,
+                         'application/javascript')
+
+    def test_render_to_json(self):
+        renderer_factory = self._makeOne()
+        renderer = renderer_factory(None)
+        request = testing.DummyRequest()
+        result = renderer({'a':'1'}, {'request':request})
+        self.assertEqual(result, '{"a": "1"}')
+        self.assertEqual(request.response.content_type,
+                         'application/json')
+
+
 class Dummy:
     pass
 
