@@ -21,8 +21,8 @@ class TestRoute(unittest.TestCase):
         self.assertEqual(route.path, ':path')
         self.assertEqual(route.name, 'name')
         self.assertEqual(route.factory, 'factory')
-        self.failUnless(route.generate.__class__ is types.FunctionType)
-        self.failUnless(route.match.__class__ is types.FunctionType)
+        self.assertTrue(route.generate.__class__ is types.FunctionType)
+        self.assertTrue(route.match.__class__ is types.FunctionType)
 
     def test_ctor_defaults(self):
         import types
@@ -31,8 +31,8 @@ class TestRoute(unittest.TestCase):
         self.assertEqual(route.path, ':path')
         self.assertEqual(route.name, 'name')
         self.assertEqual(route.factory, None)
-        self.failUnless(route.generate.__class__ is types.FunctionType)
-        self.failUnless(route.match.__class__ is types.FunctionType)
+        self.assertTrue(route.generate.__class__ is types.FunctionType)
+        self.assertTrue(route.match.__class__ is types.FunctionType)
 
     def test_match(self):
         route = self._makeOne('name', ':path')
@@ -90,6 +90,29 @@ class RoutesMapperTests(unittest.TestCase):
         self.assertEqual(mapper.routelist[0].pattern,
                          'archives/:action/:article2')
 
+    def test_connect_static(self):
+        mapper = self._makeOne()
+        mapper.connect('foo', 'archives/:action/:article', static=True)
+        self.assertEqual(len(mapper.routelist), 0)
+        self.assertEqual(len(mapper.routes), 1)
+        self.assertEqual(mapper.routes['foo'].pattern,
+                         'archives/:action/:article')
+
+    def test_connect_static_overridden(self):
+        mapper = self._makeOne()
+        mapper.connect('foo', 'archives/:action/:article', static=True)
+        self.assertEqual(len(mapper.routelist), 0)
+        self.assertEqual(len(mapper.routes), 1)
+        self.assertEqual(mapper.routes['foo'].pattern,
+                         'archives/:action/:article')
+        mapper.connect('foo', 'archives/:action/:article2')
+        self.assertEqual(len(mapper.routelist), 1)
+        self.assertEqual(len(mapper.routes), 1)
+        self.assertEqual(mapper.routes['foo'].pattern,
+                         'archives/:action/:article2')
+        self.assertEqual(mapper.routelist[0].pattern,
+                         'archives/:action/:article2')
+
     def test___call__route_matches(self):
         mapper = self._makeOne()
         mapper.connect('foo', 'archives/:action/:article')
@@ -175,6 +198,14 @@ class RoutesMapperTests(unittest.TestCase):
         self.assertEqual(result['route'], mapper.routes['root'])
         self.assertEqual(result['match'], {})
 
+    def test___call__root_route_when_path_info_notempty(self):
+        mapper = self._makeOne()
+        mapper.connect('root', '/')
+        request = self._getRequest(PATH_INFO='/')
+        result = mapper(request)
+        self.assertEqual(result['route'], mapper.routes['root'])
+        self.assertEqual(result['match'], {})
+
     def test___call__no_path_info(self):
         mapper = self._makeOne()
         mapper.connect('root', '/')
@@ -241,7 +272,8 @@ class TestCompileRoute(unittest.TestCase):
             {'baz':1, 'buz':2, 'traverse':u'/a/b'}), '/foo/1/biz/2/bar/a/b')
     
     def test_with_bracket_star(self):
-        matcher, generator = self._callFUT('/foo/{baz}/biz/{buz}/bar{remainder:.*}')
+        matcher, generator = self._callFUT(
+            '/foo/{baz}/biz/{buz}/bar{remainder:.*}')
         self.assertEqual(matcher('/foo/baz/biz/buz/bar'),
                          {'baz':'baz', 'buz':'buz', 'remainder':''})
         self.assertEqual(matcher('/foo/baz/biz/buz/bar/everything/else/here'),
@@ -268,9 +300,41 @@ class TestCompileRoute(unittest.TestCase):
         self.assertEqual(matcher('/foo/baz/biz/buz.bar'),
                          {'baz':'baz', 'buz':'buz', 'bar':'bar'})
         self.assertEqual(matcher('foo/baz/biz/buz/bar'), None)
-        self.assertEqual(generator({'baz':1, 'buz':2, 'bar': 'html'}), '/foo/1/biz/2.html')
+        self.assertEqual(generator({'baz':1, 'buz':2, 'bar': 'html'}),
+                         '/foo/1/biz/2.html')
 
-class TestCompileRouteMatchFunctional(unittest.TestCase):
+    def test_mixed_newstyle_oldstyle_pattern_defaults_to_newstyle(self):
+        # pattern: '\\/foo\\/(?P<baz>abc)\\/biz\\/(?P<buz>[^/]+)\\/bar$'
+        # note presence of :abc in pattern (oldstyle match)
+        matcher, generator = self._callFUT('foo/{baz:abc}/biz/{buz}/bar')
+        self.assertEqual(matcher('/foo/abc/biz/buz/bar'),
+                         {'baz':'abc', 'buz':'buz'})
+        self.assertEqual(generator({'baz':1, 'buz':2}), '/foo/1/biz/2/bar')
+
+    def test_custom_regex_with_embedded_squigglies(self):
+        matcher, generator = self._callFUT('/{buz:\d{4}}')
+        self.assertEqual(matcher('/2001'), {'buz':'2001'})
+        self.assertEqual(matcher('/200'), None)
+        self.assertEqual(generator({'buz':2001}), '/2001')
+
+    def test_custom_regex_with_embedded_squigglies2(self):
+        matcher, generator = self._callFUT('/{buz:\d{3,4}}')
+        self.assertEqual(matcher('/2001'), {'buz':'2001'})
+        self.assertEqual(matcher('/200'), {'buz':'200'})
+        self.assertEqual(matcher('/20'), None)
+        self.assertEqual(generator({'buz':2001}), '/2001')
+
+    def test_custom_regex_with_embedded_squigglies3(self):
+        matcher, generator = self._callFUT('/{buz:(\d{2}|\d{4})-[a-zA-Z]{3,4}-\d{2}}')
+        self.assertEqual(matcher('/2001-Nov-15'), {'buz':'2001-Nov-15'})
+        self.assertEqual(matcher('/99-June-10'), {'buz':'99-June-10'})
+        self.assertEqual(matcher('/2-Nov-15'), None)
+        self.assertEqual(matcher('/200-Nov-15'), None)
+        self.assertEqual(matcher('/2001-No-15'), None)
+        self.assertEqual(generator({'buz':'2001-Nov-15'}), '/2001-Nov-15')
+        self.assertEqual(generator({'buz':'99-June-10'}), '/99-June-10')
+
+class TestCompileRouteFunctional(unittest.TestCase):
     def matches(self, pattern, path, expected):
         from pyramid.urldispatch import _compile_route
         matcher = _compile_route(pattern)[0]
@@ -281,16 +345,41 @@ class TestCompileRouteMatchFunctional(unittest.TestCase):
         from pyramid.urldispatch import _compile_route
         self.assertEqual(_compile_route(pattern)[1](dict), result)
 
-    def test_matcher_functional(self):
+    def test_matcher_functional_notdynamic(self):
         self.matches('/', '', None)
         self.matches('', '', None)
         self.matches('/', '/foo', None)
         self.matches('/foo/', '/foo', None)
+        self.matches('', '/', {})
+        self.matches('/', '/', {})
+
+    def test_matcher_functional_newstyle(self):
+        self.matches('/{x}', '', None)
+        self.matches('/{x}', '/', None)
+        self.matches('/abc/{def}', '/abc/', None)
+        self.matches('/{x}', '/a', {'x':'a'})
+        self.matches('zzz/{x}', '/zzz/abc', {'x':'abc'})
+        self.matches('zzz/{x}*traverse', '/zzz/abc', {'x':'abc', 'traverse':()})
+        self.matches('zzz/{x}*traverse', '/zzz/abc/def/g',
+                     {'x':'abc', 'traverse':('def', 'g')})
+        self.matches('*traverse', '/zzz/abc', {'traverse':('zzz', 'abc')})
+        self.matches('*traverse', '/zzz/%20abc', {'traverse':('zzz', ' abc')})
+        self.matches('{x}', '/La%20Pe%C3%B1a', {'x':u'La Pe\xf1a'})
+        self.matches('*traverse', '/La%20Pe%C3%B1a/x',
+                     {'traverse':(u'La Pe\xf1a', 'x')})
+        self.matches('/foo/{id}.html', '/foo/bar.html', {'id':'bar'})
+        self.matches('/{num:[0-9]+}/*traverse', '/555/abc/def',
+                     {'num':'555', 'traverse':('abc', 'def')})
+        self.matches('/{num:[0-9]*}/*traverse', '/555/abc/def',
+                     {'num':'555', 'traverse':('abc', 'def')})
+        self.matches('zzz/{_}', '/zzz/abc', {'_':'abc'})
+        self.matches('zzz/{_abc}', '/zzz/abc', {'_abc':'abc'})
+        self.matches('zzz/{abc_def}', '/zzz/abc', {'abc_def':'abc'})
+
+    def test_matcher_functional_oldstyle(self):
         self.matches('/:x', '', None)
         self.matches('/:x', '/', None)
         self.matches('/abc/:def', '/abc/', None)
-        self.matches('', '/', {})
-        self.matches('/', '/', {})
         self.matches('/:x', '/a', {'x':'a'})
         self.matches('zzz/:x', '/zzz/abc', {'x':'abc'})
         self.matches('zzz/:x*traverse', '/zzz/abc', {'x':'abc', 'traverse':()})
@@ -302,10 +391,36 @@ class TestCompileRouteMatchFunctional(unittest.TestCase):
         self.matches('*traverse', '/La%20Pe%C3%B1a/x',
                      {'traverse':(u'La Pe\xf1a', 'x')})
         self.matches('/foo/:id.html', '/foo/bar.html', {'id':'bar'})
-        
-    def test_generator_functional(self):
+        self.matches('/foo/:id_html', '/foo/bar_html', {'id_html':'bar_html'})
+        self.matches('zzz/:_', '/zzz/abc', {'_':'abc'})
+        self.matches('zzz/:_abc', '/zzz/abc', {'_abc':'abc'})
+        self.matches('zzz/:abc_def', '/zzz/abc', {'abc_def':'abc'})
+
+    def test_generator_functional_notdynamic(self):
         self.generates('', {}, '/')
         self.generates('/', {}, '/')
+
+    def test_generator_functional_newstyle(self):
+        self.generates('/{x}', {'x':''}, '/')
+        self.generates('/{x}', {'x':'a'}, '/a')
+        self.generates('zzz/{x}', {'x':'abc'}, '/zzz/abc')
+        self.generates('zzz/{x}*traverse', {'x':'abc', 'traverse':''},
+                       '/zzz/abc')
+        self.generates('zzz/{x}*traverse', {'x':'abc', 'traverse':'/def/g'},
+                       '/zzz/abc/def/g')
+        self.generates('/{x}', {'x':unicode('/La Pe\xc3\xb1a', 'utf-8')},
+                       '/%2FLa%20Pe%C3%B1a')
+        self.generates('/{x}*y', {'x':unicode('/La Pe\xc3\xb1a', 'utf-8'),
+                                 'y':'/rest/of/path'},
+                       '/%2FLa%20Pe%C3%B1a/rest/of/path')
+        self.generates('*traverse', {'traverse':('a', u'La Pe\xf1a')},
+                       '/a/La%20Pe%C3%B1a')
+        self.generates('/foo/{id}.html', {'id':'bar'}, '/foo/bar.html')
+        self.generates('/foo/{_}', {'_':'20'}, '/foo/20')
+        self.generates('/foo/{_abc}', {'_abc':'20'}, '/foo/20')
+        self.generates('/foo/{abc_def}', {'abc_def':'20'}, '/foo/20')
+        
+    def test_generator_functional_oldstyle(self):
         self.generates('/:x', {'x':''}, '/')
         self.generates('/:x', {'x':'a'}, '/a')
         self.generates('zzz/:x', {'x':'abc'}, '/zzz/abc')
@@ -321,6 +436,9 @@ class TestCompileRouteMatchFunctional(unittest.TestCase):
         self.generates('*traverse', {'traverse':('a', u'La Pe\xf1a')},
                        '/a/La%20Pe%C3%B1a')
         self.generates('/foo/:id.html', {'id':'bar'}, '/foo/bar.html')
+        self.generates('/foo/:_', {'_':'20'}, '/foo/20')
+        self.generates('/foo/:_abc', {'_abc':'20'}, '/foo/20')
+        self.generates('/foo/:abc_def', {'abc_def':'20'}, '/foo/20')
 
 class DummyContext(object):
     """ """
