@@ -1,14 +1,15 @@
 import unittest
 import sys
 
-from pyramid.testing import cleanUp
+from pyramid.testing import setUp
+from pyramid.testing import tearDown
 
 class BaseTest(object):
     def setUp(self):
-        cleanUp()
+        self.config = setUp()
 
     def tearDown(self):
-        cleanUp()
+        tearDown()
 
     def _registerView(self, reg, app, name):
         from pyramid.interfaces import IRequest
@@ -24,6 +25,7 @@ class BaseTest(object):
             'SERVER_NAME':'localhost',
             'SERVER_PORT':'8080',
             'REQUEST_METHOD':'GET',
+            'PATH_INFO':'/',
             }
         environ.update(extras)
         return environ
@@ -142,6 +144,18 @@ class RenderViewToIterableTests(BaseTest, unittest.TestCase):
                                  secure=False)
         self.assertEqual(iterable, ['anotherview'])
 
+    def test_call_request_has_no_registry(self):
+        request = self._makeRequest()
+        del request.registry
+        registry = self.config.registry
+        context = self._makeContext()
+        response = DummyResponse()
+        view = make_view(response)
+        self._registerView(registry, view, 'registered')
+        iterable = self._callFUT(context, request, name='registered',
+                                 secure=True)
+        self.assertEqual(iterable, ())
+
 class RenderViewTests(BaseTest, unittest.TestCase):
     def _callFUT(self, *arg, **kw):
         from pyramid.view import render_view
@@ -184,6 +198,14 @@ class RenderViewTests(BaseTest, unittest.TestCase):
         self.assertEqual(s, 'anotherview')
 
 class TestIsResponse(unittest.TestCase):
+    def setUp(self):
+        from zope.deprecation import __show__
+        __show__.off()
+
+    def tearDown(self):
+        from zope.deprecation import __show__
+        __show__.on()
+        
     def _callFUT(self, *arg, **kw):
         from pyramid.view import is_response
         return is_response(*arg, **kw)
@@ -222,10 +244,10 @@ class TestIsResponse(unittest.TestCase):
 
 class TestViewConfigDecorator(unittest.TestCase):
     def setUp(self):
-        cleanUp()
+        setUp()
 
     def tearDown(self):
-        cleanUp()
+        tearDown()
 
     def _getTargetClass(self):
         from pyramid.view import view_config
@@ -240,6 +262,14 @@ class TestViewConfigDecorator(unittest.TestCase):
         self.assertEqual(decorator.request_type, None)
         self.assertEqual(decorator.context, None)
         self.assertEqual(decorator.permission, None)
+
+    def test_create_context_trumps_for(self):
+        decorator = self._makeOne(context='123', for_='456')
+        self.assertEqual(decorator.context, '123')
+
+    def test_create_for_trumps_context_None(self):
+        decorator = self._makeOne(context=None, for_='456')
+        self.assertEqual(decorator.context, '456')
         
     def test_create_nondefaults(self):
         decorator = self._makeOne(name=None, request_type=None, for_=None,
@@ -258,8 +288,9 @@ class TestViewConfigDecorator(unittest.TestCase):
         decorator.venusian = venusian
         def foo(): pass
         wrapped = decorator(foo)
-        self.failUnless(wrapped is foo)
-        settings = call_venusian(venusian)
+        self.assertTrue(wrapped is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(len(settings), 1)
         self.assertEqual(settings[0]['permission'], None)
         self.assertEqual(settings[0]['context'], None)
@@ -272,12 +303,30 @@ class TestViewConfigDecorator(unittest.TestCase):
         decorator.venusian.info.scope = 'class'
         class foo(object): pass
         wrapped = decorator(foo)
-        self.failUnless(wrapped is foo)
-        settings = call_venusian(venusian)
+        self.assertTrue(wrapped is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(len(settings), 1)
         self.assertEqual(settings[0]['permission'], None)
         self.assertEqual(settings[0]['context'], None)
         self.assertEqual(settings[0]['request_type'], None)
+        self.assertEqual(settings[0]['attr'], 'foo')
+
+    def test_call_class_attr_already_set(self):
+        decorator = self._makeOne(attr='abc')
+        venusian = DummyVenusian()
+        decorator.venusian = venusian
+        decorator.venusian.info.scope = 'class'
+        class foo(object): pass
+        wrapped = decorator(foo)
+        self.assertTrue(wrapped is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
+        self.assertEqual(len(settings), 1)
+        self.assertEqual(settings[0]['permission'], None)
+        self.assertEqual(settings[0]['context'], None)
+        self.assertEqual(settings[0]['request_type'], None)
+        self.assertEqual(settings[0]['attr'], 'abc')
 
     def test_stacking(self):
         decorator1 = self._makeOne(name='1')
@@ -289,14 +338,14 @@ class TestViewConfigDecorator(unittest.TestCase):
         def foo(): pass
         wrapped1 = decorator1(foo)
         wrapped2 = decorator2(wrapped1)
-        self.failUnless(wrapped1 is foo)
-        self.failUnless(wrapped2 is foo)
-        settings1 = call_venusian(venusian1)
-        self.assertEqual(len(settings1), 1)
-        self.assertEqual(settings1[0]['name'], '1')
-        settings2 = call_venusian(venusian2)
-        self.assertEqual(len(settings2), 1)
-        self.assertEqual(settings2[0]['name'], '2')
+        self.assertTrue(wrapped1 is foo)
+        self.assertTrue(wrapped2 is foo)
+        config1 = call_venusian(venusian1)
+        self.assertEqual(len(config1.settings), 1)
+        self.assertEqual(config1.settings[0]['name'], '1')
+        config2 = call_venusian(venusian2)
+        self.assertEqual(len(config2.settings), 1)
+        self.assertEqual(config2.settings[0]['name'], '2')
 
     def test_call_as_method(self):
         decorator = self._makeOne()
@@ -308,7 +357,8 @@ class TestViewConfigDecorator(unittest.TestCase):
         class foo(object):
             foomethod = decorator(foo)
             barmethod = decorator(bar)
-        settings = call_venusian(venusian)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(len(settings), 2)
         self.assertEqual(settings[0]['attr'], 'foo')
         self.assertEqual(settings[1]['attr'], 'bar')
@@ -319,8 +369,9 @@ class TestViewConfigDecorator(unittest.TestCase):
         decorator.venusian = venusian
         def foo(context, request): pass
         decorated = decorator(foo)
-        self.failUnless(decorated is foo)
-        settings = call_venusian(venusian)
+        self.assertTrue(decorated is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(settings[0]['custom_predicates'], (1,))
 
     def test_call_with_renderer_string(self):
@@ -330,24 +381,47 @@ class TestViewConfigDecorator(unittest.TestCase):
         decorator.venusian = venusian
         def foo(): pass
         wrapped = decorator(foo)
-        self.failUnless(wrapped is foo)
-        settings = call_venusian(venusian)
+        self.assertTrue(wrapped is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(len(settings), 1)
         renderer = settings[0]['renderer']
-        self.assertEqual(renderer.name, 'fixtures/minimal.pt')
-        self.assertEqual(renderer.package, pyramid.tests)
-        self.assertEqual(renderer.registry.__class__, DummyRegistry)
+        self.assertEqual(renderer, 'fixtures/minimal.pt')
+        self.assertEqual(config.pkg, pyramid.tests)
 
     def test_call_with_renderer_dict(self):
+        import pyramid.tests
         decorator = self._makeOne(renderer={'a':1})
         venusian = DummyVenusian()
         decorator.venusian = venusian
         def foo(): pass
         wrapped = decorator(foo)
-        self.failUnless(wrapped is foo)
-        settings = call_venusian(venusian)
+        self.assertTrue(wrapped is foo)
+        config = call_venusian(venusian)
+        settings = config.settings
         self.assertEqual(len(settings), 1)
         self.assertEqual(settings[0]['renderer'], {'a':1})
+        self.assertEqual(config.pkg, pyramid.tests)
+
+    def test_call_with_renderer_IRendererInfo(self):
+        import pyramid.tests
+        from pyramid.interfaces import IRendererInfo
+        class DummyRendererHelper(object):
+            implements(IRendererInfo)
+        renderer_helper = DummyRendererHelper()
+        decorator = self._makeOne(renderer=renderer_helper)
+        venusian = DummyVenusian()
+        decorator.venusian = venusian
+        def foo(): pass
+        wrapped = decorator(foo)
+        self.assertTrue(wrapped is foo)
+        context = DummyVenusianContext()
+        config = call_venusian(venusian, context)
+        settings = config.settings
+        self.assertEqual(len(settings), 1)
+        renderer = settings[0]['renderer']
+        self.assertTrue(renderer is renderer_helper)
+        self.assertEqual(config.pkg, pyramid.tests)
 
 class Test_append_slash_notfound_view(BaseTest, unittest.TestCase):
     def _callFUT(self, context, request):
@@ -398,6 +472,14 @@ class Test_append_slash_notfound_view(BaseTest, unittest.TestCase):
         response = self._callFUT(context, request)
         self.assertEqual(response.status, '404 Not Found')
 
+    def test_no_route_matches(self):
+        request = self._makeRequest(PATH_INFO='/abc')
+        context = ExceptionResponse()
+        mapper = self._registerMapper(request.registry, True)
+        mapper.routelist[0].val = None
+        response = self._callFUT(context, request)
+        self.assertEqual(response.status, '404 Not Found')
+
     def test_matches(self):
         request = self._makeRequest(PATH_INFO='/abc')
         context = ExceptionResponse()
@@ -406,6 +488,14 @@ class Test_append_slash_notfound_view(BaseTest, unittest.TestCase):
         self.assertEqual(response.status, '302 Found')
         self.assertEqual(response.location, '/abc/')
 
+    def test_matches_with_script_name(self):
+        request = self._makeRequest(PATH_INFO='/abc', SCRIPT_NAME='/foo')
+        context = ExceptionResponse()
+        self._registerMapper(request.registry, True)
+        response = self._callFUT(context, request)
+        self.assertEqual(response.status, '302 Found')
+        self.assertEqual(response.location, '/foo/abc/')
+
     def test_with_query_string(self):
         request = self._makeRequest(PATH_INFO='/abc', QUERY_STRING='a=1&b=2')
         context = ExceptionResponse()
@@ -436,13 +526,20 @@ class Test_default_exceptionresponse_view(unittest.TestCase):
     def test_is_exception(self):
         context = Exception()
         result = self._callFUT(context, None)
-        self.failUnless(result is context)
+        self.assertTrue(result is context)
+
+    def test_is_not_exception_context_is_false_still_chose(self):
+        request = DummyRequest()
+        request.exception = 0
+        result = self._callFUT(None, request)
+        self.assertTrue(result is None)
 
     def test_is_not_exception_no_request_exception(self):
         context = object()
         request = DummyRequest()
+        request.exception = None
         result = self._callFUT(context, request)
-        self.failUnless(result is context)
+        self.assertTrue(result is context)
 
     def test_is_not_exception_request_exception(self):
         context = object()
@@ -451,6 +548,24 @@ class Test_default_exceptionresponse_view(unittest.TestCase):
         result = self._callFUT(context, request)
         self.assertEqual(result, 'abc')
 
+class Test_static(unittest.TestCase):
+    def setUp(self):
+        from zope.deprecation import __show__
+        __show__.off()
+
+    def tearDown(self):
+        from zope.deprecation import __show__
+        __show__.on()
+
+    def _makeOne(self, path, package_name):
+        from pyramid.view import static
+        return static(path, package_name)
+        
+    def test_it(self):
+        path = 'fixtures'
+        view = self._makeOne(path, None)
+        self.assertEqual(view.docroot, 'fixtures')
+
 class ExceptionResponse(Exception):
     status = '404 Not Found'
     app_iter = ['Not Found']
@@ -467,9 +582,20 @@ def make_view(response):
 class DummyRequest:
     exception = None
 
-class DummyResponse:
-    status = '200 OK'
+    def __init__(self, environ=None):
+        if environ is None:
+            environ = {}
+        self.environ = environ
+        
+from pyramid.interfaces import IResponse
+from zope.interface import implements
+
+class DummyResponse(object):
+    implements(IResponse)
     headerlist = ()
+    app_iter = ()
+    status = '200 OK'
+    environ = None
     def __init__(self, body=None):
         if body is None:
             self.app_iter = ()
@@ -507,13 +633,18 @@ class DummyConfig(object):
     def add_view(self, **kw):
         self.settings.append(kw)
 
+    def with_package(self, pkg):
+        self.pkg = pkg
+        return self
+
 class DummyVenusianContext(object):
     def __init__(self):
         self.config = DummyConfig()
         
-def call_venusian(venusian):
-    context = DummyVenusianContext()
+def call_venusian(venusian, context=None):
+    if context is None:
+        context = DummyVenusianContext()
     for wrapped, callback, category in venusian.attachments:
         callback(context, None, None)
-    return context.config.settings
+    return context.config
 
